searchData={"items":[{"type":"module","title":"Pundit","doc":"Pundit provides a set of helpers which guide you in leveraging regular Elixir methods to\nbuild a simple authorization system.  This library is based heavily on Jonas Nicklas'\n[Ruby project of the same name](https://github.com/varvet/pundit).\n\nSimple Elixir functions are defined for a given struct and allow you to encapsulate authorization logic.  You can use\nthis code within a module that is an `Ecto.Schema`, but that's not necessary.  The action names are taken from the list\nof [actions defined by Phoenix controllers](https://hexdocs.pm/phoenix/controllers.html#actions).","ref":"Pundit.html"},{"type":"module","title":"Examples - Pundit","doc":"Here's a basic example, starting with a simple struct for a `Post`.  A module named `Post.Policy` should be created to\nencapsulate all of the access methods (`Pundit` will automatically look for the ` .Policy` module\nto determine the module name to look at for access methods).\n\nTo declare an initial set of access functions (`show?`, `edit?`, `delete?`, etc)\nwhich all return `false` (default safe!), just `use` `Pundit.DefaultPolicy`.  You can then override the functions as needed\nwith the logic necessary to determine whether a user should be able to perform the given action.  In this example, we only\ndetermine whether a user can `edit?` a post, leaving all other functions (like `delete?`) to return the default of `false`.\n\n    defmodule Post do\n      defstruct [:author, :title, :body, :comments]\n\n      defmodule Policy do\n        # This will initialize all the functions listed below, that all return false\n        # by default. Override them individually to return true when they should.\n        use Pundit.DefaultPolicy\n\n        def edit?(post, user) do\n          user.name == post.author\n        end\n      end\n    end\n\n    post = %Post{author: \"Snake Plissken\"}\n    author = %{name: \"Snake Plissken\"}\n    # next line is same as Pundit.can?(post, author, :edit?)\n    if Pundit.edit?(post, author) do\n      IO.puts(\"Can edit!\")\n    end\n\n    if Pundit.delete?(post, author) do\n      IO.puts(\"This line should never be called\")\n    end\n\n    # raise exception if user should be able to do a thing\n    Pundit.authorize!(post, author, :edit?)","ref":"Pundit.html#module-examples"},{"type":"module","title":"Scope - Pundit","doc":"You can also provide query scope for a struct (say, if you're using `Ecto.Schema`) for a given user.  For instance,\nsay our `Post` was an `Ecto` schema.  Our function for scoping all `Post`s to a specific `User` could be to find all\n`Post`s that were authored by a user.  For instance:\n\n    defmodule Post do\n      use Ecto.Schema\n      import Ecto.Query, only: [from: 2]\n\n      defmodule Policy do\n        use Pundit.DefaultPolicy\n\n        def scope(query, user) do\n          from post in query,\n            where: post.author_id == ^user.id\n        end\n      end\n    end\n\n    user = MyApp.Repo.get(User, 1)\n    posts = Pundit.scope(Post, user) |> Repo.all()\n\n    query = from p in Post, where: p.comment_count > 10\n    popular_posts = Pundit.scope(query, user) |> Repo.all()","ref":"Pundit.html#module-scope"},{"type":"function","title":"Pundit.authorize/3","doc":"Return a tuple based on whether a user can perform the action on the thing.\n\nReturns `{:ok}` if a user can perform the action, or `{:error, message}` if not.","ref":"Pundit.html#authorize/3"},{"type":"function","title":"Pundit.authorize!/3","doc":"Raise a `Pundit.NotAuthorizedError` exception unless the user can perform the action on the thing.","ref":"Pundit.html#authorize!/3"},{"type":"function","title":"Pundit.can?/3","doc":"Determine if a use can perform an action on a given thing.\n\nThis will attempt to call a function with the same name as the action on the policy\nmodule of the given thing.  For instance:\n\n    Pundit.can?(post, user, :edit?)\n\nis the same as:\n\n    Post.Policy.edit?(post, user)","ref":"Pundit.html#can?/3"},{"type":"function","title":"Pundit.create?/2","doc":"Returns true only if the user should be allowed to create a new kind of thing.","ref":"Pundit.html#create?/2"},{"type":"function","title":"Pundit.delete?/2","doc":"Returns true only if the user should be allowed to delete a thing.","ref":"Pundit.html#delete?/2"},{"type":"function","title":"Pundit.edit?/2","doc":"Returns true only if the user should be allowed to see a form for updating the thing.\n\nSee [the page on Phoenix controllers](https://hexdocs.pm/phoenix/controllers.html#actions) for more details on the\npurpose of this action.","ref":"Pundit.html#edit?/2"},{"type":"function","title":"Pundit.index?/2","doc":"Returns true only if the user should be allowed to see an index (list) of the given things.","ref":"Pundit.html#index?/2"},{"type":"function","title":"Pundit.new?/2","doc":"Returns true only if the user should be allowed to see a form to create a new thing.\n\nSee [the page on Phoenix controllers](https://hexdocs.pm/phoenix/controllers.html#actions) for more details on the\npurpose of this action.","ref":"Pundit.html#new?/2"},{"type":"function","title":"Pundit.scope/2","doc":"Scope a `Ecto.Query` or `Ecto.Schema` to a given user.\n\nThis will call the function `scope` on the policy module of the given thing.  For instance:\n\n    Pundit.scope(Post, user)\n\nis the same as:\n\n    Post.Policy.scope(Post, user)\n\nHere's an example with a `Ecto.Query`:\n\n    query = from post in Post, where: post.comments > 10\n    # This call...\n    Pundit.scope(query, user)\n    # Is the same as...\n    Post.Policy.scope(query, user)\n\nThis is just helpful shorthand.","ref":"Pundit.html#scope/2"},{"type":"function","title":"Pundit.show?/2","doc":"Returns true only if the user should be allowed to see the given thing.","ref":"Pundit.html#show?/2"},{"type":"function","title":"Pundit.update?/2","doc":"Returns true only if the user should be allowed to update the attributes of a thing.","ref":"Pundit.html#update?/2"},{"type":"behaviour","title":"Pundit.DefaultPolicy","doc":"Default access policies for a given type.\n\nAll of the functions here are named for actions in a [Phoenix controller](https://hexdocs.pm/phoenix/controllers.html#actions).\n\nIf you `use` this module, then default implementations will be added in your module that all \nreturn `false` by default (default safe, nothing is permitted).  All are overrideable.","ref":"Pundit.DefaultPolicy.html"},{"type":"callback","title":"Pundit.DefaultPolicy.create?/2","doc":"Returns true only if the user should be allowed to create a new kind of thing.","ref":"Pundit.DefaultPolicy.html#c:create?/2"},{"type":"callback","title":"Pundit.DefaultPolicy.delete?/2","doc":"Returns true only if the user should be allowed to delete a thing.","ref":"Pundit.DefaultPolicy.html#c:delete?/2"},{"type":"callback","title":"Pundit.DefaultPolicy.edit?/2","doc":"Returns true only if the user should be allowed to see a form for updating the thing.\n\nSee [the page on Phoenix controllers](https://hexdocs.pm/phoenix/controllers.html#actions) for more details on the\npurpose of this action.","ref":"Pundit.DefaultPolicy.html#c:edit?/2"},{"type":"callback","title":"Pundit.DefaultPolicy.index?/2","doc":"Returns true only if the user should be allowed to see an index (list) of the given things.","ref":"Pundit.DefaultPolicy.html#c:index?/2"},{"type":"callback","title":"Pundit.DefaultPolicy.new?/2","doc":"Returns true only if the user should be allowed to see a form to create a new thing.\n\nSee [the page on Phoenix controllers](https://hexdocs.pm/phoenix/controllers.html#actions) for more details on the\npurpose of this action.","ref":"Pundit.DefaultPolicy.html#c:new?/2"},{"type":"callback","title":"Pundit.DefaultPolicy.show?/2","doc":"Returns true only if the user should be allowed to see the given thing.","ref":"Pundit.DefaultPolicy.html#c:show?/2"},{"type":"callback","title":"Pundit.DefaultPolicy.update?/2","doc":"Returns true only if the user should be allowed to update the attributes of a thing.","ref":"Pundit.DefaultPolicy.html#c:update?/2"},{"type":"exception","title":"Pundit.NotAuthorizedError","doc":"Exception raised when a user attempts to perform an action they're not authorized to perform.","ref":"Pundit.NotAuthorizedError.html"},{"type":"exception","title":"Pundit.NotDefinedError","doc":"Exception raised when a module doesn't implement a necessary access function.","ref":"Pundit.NotDefinedError.html"},{"type":"extras","title":"Pundit","doc":"# Pundit\n[![Build Status](https://secure.travis-ci.org/bmuller/pundit-elixir.png?branch=master)](https://travis-ci.org/bmuller/pundit-elixir)\n[![Hex pm](http://img.shields.io/hexpm/v/pundit.svg?style=flat)](https://hex.pm/packages/pundit)\n[![API Docs](https://img.shields.io/badge/api-docs-lightgreen.svg?style=flat)](https://hexdocs.pm/pundit/)\n\nPundit provides a set of helpers which guide you in leveraging regular Elixir methods to\nbuild a simple authorization system.  This library is based heavily on Jonas Nicklas' [Ruby project of the same name](https://github.com/varvet/pundit).\n\nSimple Elixir functions are defined for a given struct and allow you to encapsulate authorization logic.  You can use\nthis code within a module that is an Ecto.Schema, but that's not necessary (Ecto isn't required).  The action names are taken from the list\nof [actions defined by Phoenix controllers](https://hexdocs.pm/phoenix/controllers.html#actions).","ref":"readme.html"},{"type":"extras","title":"Installation - Pundit","doc":"To install Pundit, just add an entry to your `mix.exs`:\n\n``` elixir\ndef deps do\n  [\n    # ...\n    {:pundit, \"~> 1.0\"}\n  ]\nend\n```\n\n(Check [Hex](https://hex.pm/packages/pundit) to make sure you're using an up-to-date version number.)","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Pundit","doc":"Here's a basic example, starting with a simple struct for a `Post`.  A module named `Post.Policy` should be created to\nencapsulate all of the access methods (`Pundit` will automatically look for the ` .Policy` module\nto determine the module name to look at for access methods).\n\nTo declare an initial set of access functions (`show?`, `edit?`, `delete?`, etc)\nwhich all return `false` (default safe!), just `use` `Pundit.DefaultPolicy`.  You can then override the functions as needed\nwith the logic necessary to determine whether a user should be able to perform the given action.  In this example, we only\ndetermine whether a user can `edit?` a post, leaving all other functions (like `delete?`) to return the default of `false`.\n\n```elixir\ndefmodule Post do\n  defstruct [:author, :title, :body, :comments]\n\n  defmodule Policy do\n    # This will initialize all the action functions, all of which return false\n    # by default. Override them individually to return true when they should,\n    # like edit? is overriden below.\n    use Pundit.DefaultPolicy\n\n    def edit?(post, user) do\n      user.name == post.author\n    end\n  end\nend\n\npost = %Post{author: \"Snake Plissken\"}\nauthor = %{name: \"Snake Plissken\"}\n# next line is same as Pundit.can?(post, author, :edit?)\n# Pundit will just delegate to Post.Policy.edit?(post, user)\nif Pundit.edit?(post, author) do\n  IO.puts(\"Can edit!\")\nend\n\nif Pundit.delete?(post, author) do\n  IO.puts(\"This line should never be called\")\nend\n\n# raise exception if user should be able to do a thing\nPundit.authorize!(post, author, :edit?)\n```","ref":"readme.html#usage"},{"type":"extras","title":"Scope - Pundit","doc":"You can also provide query scope for a struct (say, if you're using `Ecto.Schema`) for a given user.  For instance,\nsay our `Post` was an `Ecto` schema.  Our function for scoping all `Post`s to a specific `User` could be to find all\n`Post`s that were authored by a user.  For instance:\n\n```elixir\ndefmodule Post do\n  use Ecto.Schema\n  import Ecto.Query, only: [from: 2]\n\n  defmodule Policy do\n    use Pundit.DefaultPolicy\n\n    def scope(query, user) do\n      from post in query,\n        where: post.author_id == ^user.id\n    end\n  end\nend\n\nuser = MyApp.Repo.get(User, 1)\nposts = Pundit.scope(Post, user) |> Repo.all()\n\nquery = from p in Post, where: p.comment_count > 10\npopular_posts = Pundit.scope(query, user) |> Repo.all()\n```\n\nSee [the docs](https://hexdocs.pm/pundit) for more examples.","ref":"readme.html#scope"},{"type":"extras","title":"Running Tests - Pundit","doc":"To run tests:\n\n```shell\n$ mix test\n```","ref":"readme.html#running-tests"},{"type":"extras","title":"Reporting Issues - Pundit","doc":"Please report all issues [on github](https://github.com/bmuller/pundit-elixir/issues).","ref":"readme.html#reporting-issues"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}